@@ -20,6 +20,7 @@
 #include <sys/stat.h>
 
 #include <event.h>
+#include <imsg.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -93,8 +94,8 @@ got_gotweb_dupfd(int *priv_fd, int *fd)
 {
 	*fd = dup(*priv_fd);
 
-	if (*fd < 0)
-		return NULL;
+	if (*fd == -1)
+		return got_error_from_errno("dup");
 
 	return NULL;
 }
@@ -418,15 +419,13 @@ got_get_repo_commits(struct request *c, int limit)
 			 */
 			if (chk_next && (qs->action == BRIEFS ||
 			    qs->action == COMMITS || qs->action == SUMMARY)) {
-				t->next_id = strdup(repo_commit->commit_id);
-				if (t->next_id == NULL) {
+				t->more_id = strdup(repo_commit->commit_id);
+				if (t->more_id == NULL) {
 					error = got_error_from_errno("strdup");
 					goto done;
 				}
-				if (commit) {
-					got_object_commit_close(commit);
-					commit = NULL;
-				}
+				got_object_commit_close(commit);
+				commit = NULL;
 				TAILQ_REMOVE(&t->repo_commits, repo_commit,
 				    entry);
 				gotweb_free_repo_commit(repo_commit);
@@ -769,8 +768,10 @@ got_output_repo_tree(struct request *c,
 
 	for (i = 0; i < nentries; i++) {
 		te = got_object_tree_get_entry(tree, i);
-		if (cb(c->tp, te) == -1)
+		if (cb(c->tp, te) == -1) {
+			error = got_error(GOT_ERR_CANCELLED);
 			break;
+		}
 	}
 done:
 	free(escaped_name);
@@ -893,8 +894,10 @@ got_output_blob_by_lines(struct template *tp, struct got_blob_object *blob,
 		if (err || linelen == -1)
 			break;
 		lineno++;
-		if (cb(tp, line, lineno) == -1)
+		if (cb(tp, line, lineno) == -1) {
+			err = got_error(GOT_ERR_CANCELLED);
 			break;
+		}
 	}
 
 	free(line);
@@ -983,8 +986,10 @@ got_gotweb_blame_cb(void *arg, int nlines, int lineno,
 		}
 
 		if (a->cb(c->tp, line, bline, a->nlines_prec,
-		    a->lineno_cur) == -1)
+		    a->lineno_cur) == -1) {
+			err = got_error(GOT_ERR_CANCELLED);
 			break;
+		}
 
 		a->lineno_cur++;
 		bline = &a->lines[a->lineno_cur - 1];
