@@ -174,7 +174,7 @@
 #define GOT_ERR_BAD_TAG_SIGNATURE 154
 #define GOT_ERR_VERIFY_TAG_SIGNATURE 155
 #define GOT_ERR_SIGNING_TAG	156
-#define GOT_ERR_COMMIT_REDUNDANT_AUTHOR 157
+/* 157 is currently unused */
 #define GOT_ERR_BAD_QUERYSTRING	158
 #define GOT_ERR_INTEGRATE_BRANCH 159
 #define GOT_ERR_BAD_REQUEST	160
@@ -262,6 +262,14 @@ const struct got_error *got_ferror(FILE *, int);
 struct got_object_id; /* forward declaration */
 const struct got_error *got_error_no_obj(struct got_object_id *);
 
+/*
+ * Obtain an error with code GOT_ERR_OBJ_CSUM and an error message which
+ * contains the specified object ID. The message buffer is statically
+ * allocated; future invocations of this function will overwrite the
+ * message set during earlier invocations.
+ */
+const struct got_error *got_error_checksum(struct got_object_id *);
+
 /*
  * Obtain an error with code GOT_ERR_NOT_REF and an error message which
  * contains the specified reference name. The message buffer is statically
