@@ -15,6 +15,7 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include <sys/queue.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netdb.h>
@@ -301,6 +302,7 @@ got_dial_git(int *newfd, const char *host, const char *port,
 		err = got_error_from_errno("connect");
 		close(fd);
 	}
+	freeaddrinfo(servinfo);
 	if (p == NULL)
 		goto done;
 
