@@ -52,7 +52,7 @@
 #include "got_lib_object_parse.h"
 #include "got_lib_object_create.h"
 #include "got_lib_pack.h"
-#include "got_lib_sha1.h"
+#include "got_lib_hash.h"
 #include "got_lib_privsep.h"
 #include "got_lib_object_cache.h"
 #include "got_lib_repository.h"
@@ -99,8 +99,9 @@ got_fetch_pack(struct got_object_id **pack_hash, struct got_pathlist_head *refs,
     int mirror_references, int fetch_all_branches,
     struct got_pathlist_head *wanted_branches,
     struct got_pathlist_head *wanted_refs, int list_refs_only, int verbosity,
-    int fetchfd, struct got_repository *repo,
-    got_fetch_progress_cb progress_cb, void *progress_arg)
+    int fetchfd, struct got_repository *repo, const char *worktree_refname,
+    const char *remote_head, int no_head, got_fetch_progress_cb progress_cb,
+    void *progress_arg)
 {
 	size_t i;
 	int imsg_fetchfds[2], imsg_idxfds[2];
@@ -256,7 +257,7 @@ got_fetch_pack(struct got_object_id **pack_hash, struct got_pathlist_head *refs,
 	}
 	err = got_privsep_send_fetch_req(&fetchibuf, nfetchfd, &have_refs,
 	    fetch_all_branches, wanted_branches, wanted_refs,
-	    list_refs_only, verbosity);
+	    list_refs_only, worktree_refname, remote_head, no_head, verbosity);
 	if (err != NULL)
 		goto done;
 	nfetchfd = -1;
