@@ -158,6 +158,7 @@ enum got_imsg_type {
 	GOT_IMSG_GITCONFIG_REMOTES_REQUEST,
 	GOT_IMSG_GITCONFIG_INT_VAL,
 	GOT_IMSG_GITCONFIG_STR_VAL,
+	GOT_IMSG_GITCONFIG_PAIR,
 	GOT_IMSG_GITCONFIG_REMOTES,
 	GOT_IMSG_GITCONFIG_REMOTE,
 	GOT_IMSG_GITCONFIG_OWNER_REQUEST,
@@ -220,7 +221,7 @@ struct got_imsg_error {
  * Structure for GOT_IMSG_TREE_REQUEST and GOT_IMSG_OBJECT data.
  */
 struct got_imsg_object {
-	uint8_t id[SHA1_DIGEST_LENGTH];
+	struct got_object_id id;
 
 	/* These fields are the same as in struct got_object. */
 	int type;
@@ -233,7 +234,7 @@ struct got_imsg_object {
 
 /* Structure for GOT_IMSG_COMMIT data. */
 struct got_imsg_commit_object {
-	uint8_t tree_id[SHA1_DIGEST_LENGTH];
+	struct got_object_id tree_id;
 	size_t author_len;
 	time_t author_time;
 	time_t author_gmtoff;
@@ -247,7 +248,7 @@ struct got_imsg_commit_object {
 	 * Followed by author_len + committer_len data bytes
 	 */
 
-	/* Followed by 'nparents' SHA1_DIGEST_LENGTH length strings */
+	/* Followed by 'nparents' struct got_object_id */
 
 	/*
 	 * Followed by 'logmsg_len' bytes of commit log message data in
@@ -304,7 +305,7 @@ struct got_imsg_raw_obj {
 
 /* Structure for GOT_IMSG_RAW_DELTA. */
 struct got_imsg_raw_delta {
-	uint8_t base_id[SHA1_DIGEST_LENGTH];
+	struct got_object_id base_id;
 	uint64_t base_size;
 	uint64_t result_size;
 	off_t delta_size;
@@ -343,7 +344,7 @@ struct got_imsg_reused_deltas {
 
 /* Structure for GOT_IMSG_COMMIT_PAINTING_REQUEST. */
 struct got_imsg_commit_painting_request {
-	uint8_t id[SHA1_DIGEST_LENGTH];
+	struct got_object_id id;
 	int idx;
 	int color;
 } __attribute__((__packed__));
@@ -364,7 +365,7 @@ struct got_imsg_painted_commits {
 
 /* Structure for GOT_IMSG_TAG data. */
 struct got_imsg_tag_object {
-	uint8_t id[SHA1_DIGEST_LENGTH];
+	struct got_object_id id;
 	int obj_type;
 	size_t tag_len;
 	size_t tagger_len;
@@ -384,7 +385,7 @@ struct got_imsg_tag_object {
 
 /* Structure for GOT_IMSG_FETCH_HAVE_REF data. */
 struct got_imsg_fetch_have_ref {
-	uint8_t id[SHA1_DIGEST_LENGTH];
+	struct got_object_id id;
 	size_t name_len;
 	/* Followed by name_len data bytes. */
 } __attribute__((__packed__));
@@ -403,12 +404,17 @@ struct got_imsg_fetch_wanted_ref {
 
 /* Structure for GOT_IMSG_FETCH_REQUEST data. */
 struct got_imsg_fetch_request {
+	int no_head;
 	int fetch_all_branches;
 	int list_refs_only;
 	int verbosity;
+	size_t worktree_branch_len;
+	size_t remote_head_len;
 	size_t n_have_refs;
 	size_t n_wanted_branches;
 	size_t n_wanted_refs;
+	/* Followed by worktree_branch_len bytes of reference name. */
+	/* Followed by remote_head_len bytes of reference name. */
 	/* Followed by n_have_refs GOT_IMSG_FETCH_HAVE_REF messages. */
 	/* Followed by n_wanted_branches times GOT_IMSG_FETCH_WANTED_BRANCH. */
 	/* Followed by n_wanted_refs times GOT_IMSG_FETCH_WANTED_REF. */
@@ -433,7 +439,7 @@ struct got_imsg_fetch_symrefs {
 /* Structure for GOT_IMSG_FETCH_REF data. */
 struct got_imsg_fetch_ref {
 	/* Describes a reference which will be fetched. */
-	uint8_t refid[SHA1_DIGEST_LENGTH];
+	struct got_object_id refid;
 	/* Followed by reference name in remaining data of imsg buffer. */
 };
 
@@ -458,7 +464,7 @@ struct got_imsg_send_upload_progress {
 
 /* Structure for GOT_IMSG_SEND_REF data. */
 struct got_imsg_send_ref {
-	uint8_t id[SHA1_DIGEST_LENGTH];
+	struct got_object_id id;
 	int delete;
 	size_t name_len;
 	/* Followed by name_len data bytes. */
@@ -466,7 +472,7 @@ struct got_imsg_send_ref {
 
 /* Structure for GOT_IMSG_SEND_REMOTE_REF data. */
 struct got_imsg_send_remote_ref {
-	uint8_t id[SHA1_DIGEST_LENGTH];
+	struct got_object_id id;
 	size_t name_len;
 	/* Followed by name_len data bytes. */
 } __attribute__((__packed__));
@@ -526,7 +532,7 @@ struct got_object_id;
  * GOT_IMSG_PACKED_RAW_OBJECT_REQUEST data.
  */
 struct got_imsg_packed_object {
-	uint8_t id[SHA1_DIGEST_LENGTH];
+	struct got_object_id id;
 	int idx;
 } __attribute__((__packed__));
 
@@ -546,7 +552,7 @@ struct got_imsg_delta {
  * Structure for GOT_IMSG_RAW_DELTA_REQUEST data.
  */
 struct got_imsg_raw_delta_request {
-	uint8_t id[SHA1_DIGEST_LENGTH];
+	struct got_object_id id;
 	int idx;
 };
 
@@ -579,7 +585,7 @@ struct got_imsg_commit_traversal_request {
 /* Structure for GOT_IMSG_TRAVERSED_COMMITS  */
 struct got_imsg_traversed_commits {
 	size_t ncommits;
-	/* Followed by ncommit IDs of SHA1_DIGEST_LENGTH each */
+	/* Followed by ncommit struct got_object_id */
 } __attribute__((__packed__));
 
 /* Structure for GOT_IMSG_ENUMERATED_COMMIT  */
@@ -626,6 +632,16 @@ struct got_imsg_remotes {
 	int nremotes; /* This many GOT_IMSG_GITCONFIG_REMOTE messages follow. */
 };
 
+/*
+ * Structure for GOT_IMSG_GITCONFIG_PAIR.
+ */
+struct got_imsg_gitconfig_pair {
+	size_t	klen;
+	size_t	vlen;
+	/* Followed by klen data bytes of key string. */
+	/* Followed by vlen data bytes of value string. */
+};
+
 /*
  * Structure for GOT_IMSG_PATCH data.
  */
@@ -687,7 +703,7 @@ const struct got_error *got_privsep_recv_index_progress(int *, int *, int *,
     int *, int *, struct imsgbuf *ibuf);
 const struct got_error *got_privsep_send_fetch_req(struct imsgbuf *, int,
     struct got_pathlist_head *, int, struct got_pathlist_head *,
-    struct got_pathlist_head *, int, int);
+    struct got_pathlist_head *, int, const char *, const char *, int, int);
 const struct got_error *got_privsep_send_fetch_outfd(struct imsgbuf *, int);
 const struct got_error *got_privsep_recv_fetch_progress(int *,
     struct got_object_id **, char **, struct got_pathlist_head *, char **,
@@ -747,6 +763,8 @@ const struct got_error *got_privsep_send_gitconfig_remotes_req(
 const struct got_error *got_privsep_send_gitconfig_owner_req(struct imsgbuf *);
 const struct got_error *got_privsep_recv_gitconfig_str(char **,
     struct imsgbuf *);
+const struct got_error *got_privsep_recv_gitconfig_pair(char **, char **,
+    struct imsgbuf *);
 const struct got_error *got_privsep_recv_gitconfig_int(int *, struct imsgbuf *);
 const struct got_error *got_privsep_recv_gitconfig_remotes(
     struct got_remote_repo **, int *, struct imsgbuf *);
