@@ -39,7 +39,7 @@
 #include "got_opentemp.h"
 #include "got_path.h"
 
-#include "got_lib_sha1.h"
+#include "got_lib_hash.h"
 #include "got_lib_delta.h"
 #include "got_lib_inflate.h"
 #include "got_lib_object.h"
@@ -150,7 +150,7 @@ got_object_open_by_id_str(struct got_object **obj, struct got_repository *repo,
 {
 	struct got_object_id id;
 
-	if (!got_parse_sha1_digest(id.sha1, id_str))
+	if (!got_parse_object_id(&id, id_str, GOT_HASH_SHA1))
 		return got_error_path(id_str, GOT_ERR_BAD_OBJ_ID_STR);
 
 	return got_object_open(obj, repo, &id);
@@ -374,7 +374,7 @@ got_object_blob_rewind(struct got_blob_object *blob)
 char *
 got_object_blob_id_str(struct got_blob_object *blob, char *buf, size_t size)
 {
-	return got_sha1_digest_to_str(blob->id.sha1, buf, size);
+	return got_object_id_hex(&blob->id, buf, size);
 }
 
 size_t
