@@ -102,7 +102,7 @@ done:
 			close(accumfd);
 	} else
 		pack->child_has_tempfiles = 1;
-	return NULL;
+	return err;
 }
 
 static const struct got_error *
@@ -444,7 +444,7 @@ got_object_open(struct got_object **obj, struct got_repository *repo,
 	if (err)
 		return err;
 
-	memcpy((*obj)->id.sha1, id->sha1, SHA1_DIGEST_LENGTH);
+	memcpy(&(*obj)->id, id, sizeof(*id));
 
 	(*obj)->refcnt++;
 	return got_repo_cache_object(repo, id, *obj);
@@ -1011,7 +1011,7 @@ open_blob(struct got_blob_object **blob, struct got_repository *repo,
 
 	(*blob)->hdrlen = hdrlen;
 	(*blob)->blocksize = blocksize;
-	memcpy(&(*blob)->id.sha1, id->sha1, SHA1_DIGEST_LENGTH);
+	memcpy(&(*blob)->id, id, sizeof(*id));
 
 done:
 	free(path_packfile);
