@@ -29,6 +29,7 @@
 #include <errno.h>
 #include <stdint.h>
 #include <poll.h>
+#include <imsg.h>
 #include <unistd.h>
 #include <zlib.h>
 
@@ -39,7 +40,7 @@
 #include "got_path.h"
 #include "got_repository.h"
 
-#include "got_lib_sha1.h"
+#include "got_lib_hash.h"
 #include "got_lib_delta.h"
 #include "got_lib_inflate.h"
 #include "got_lib_object.h"
@@ -364,7 +365,7 @@ got_privsep_send_commit_req(struct imsgbuf *ibuf, int fd,
 	memset(&iobj, 0, sizeof(iobj));
 	if (pack_idx != -1) { /* commit is packed */
 		iobj.idx = pack_idx;
-		memcpy(iobj.id, id->sha1, sizeof(iobj.id));
+		memcpy(&iobj.id, id, sizeof(iobj.id));
 		data = &iobj;
 		len = sizeof(iobj);
 	} else {
@@ -398,7 +399,7 @@ got_privsep_send_tree_req(struct imsgbuf *ibuf, int fd,
 	if (wbuf == NULL)
 		return got_error_from_errno("imsg_create TREE_REQUEST");
 
-	if (imsg_add(wbuf, id->sha1, SHA1_DIGEST_LENGTH) == -1)
+	if (imsg_add(wbuf, id, sizeof(*id)) == -1)
 		return got_error_from_errno("imsg_add TREE_REQUEST");
 
 	if (pack_idx != -1) { /* tree is packed */
@@ -423,7 +424,7 @@ got_privsep_send_tag_req(struct imsgbuf *ibuf, int fd,
 	memset(&iobj, 0, sizeof(iobj));
 	if (pack_idx != -1) { /* tag is packed */
 		iobj.idx = pack_idx;
-		memcpy(iobj.id, id->sha1, sizeof(iobj.id));
+		memcpy(&iobj.id, id, sizeof(iobj.id));
 		data = &iobj;
 		len = sizeof(iobj);
 	} else {
@@ -450,7 +451,7 @@ got_privsep_send_blob_req(struct imsgbuf *ibuf, int infd,
 	memset(&iobj, 0, sizeof(iobj));
 	if (pack_idx != -1) { /* blob is packed */
 		iobj.idx = pack_idx;
-		memcpy(iobj.id, id->sha1, sizeof(iobj.id));
+		memcpy(&iobj.id, id, sizeof(iobj.id));
 		data = &iobj;
 		len = sizeof(iobj);
 	} else {
@@ -510,7 +511,7 @@ got_privsep_send_obj(struct imsgbuf *ibuf, struct got_object *obj)
 
 	memset(&iobj, 0, sizeof(iobj));
 
-	memcpy(iobj.id, obj->id.sha1, sizeof(iobj.id));
+	memcpy(&iobj.id, &obj->id, sizeof(iobj.id));
 	iobj.type = obj->type;
 	iobj.flags = obj->flags;
 	iobj.hdrlen = obj->hdrlen;
@@ -531,34 +532,61 @@ const struct got_error *
 got_privsep_send_fetch_req(struct imsgbuf *ibuf, int fd,
     struct got_pathlist_head *have_refs, int fetch_all_branches,
     struct got_pathlist_head *wanted_branches,
-    struct got_pathlist_head *wanted_refs, int list_refs_only, int verbosity)
+    struct got_pathlist_head *wanted_refs, int list_refs_only,
+    const char *worktree_branch, const char *remote_head,
+    int no_head, int verbosity)
 {
 	const struct got_error *err = NULL;
 	struct ibuf *wbuf;
-	size_t len;
 	struct got_pathlist_entry *pe;
 	struct got_imsg_fetch_request fetchreq;
+	size_t remote_head_len, worktree_branch_len, len = sizeof(fetchreq);
+
+	if (worktree_branch) {
+		worktree_branch_len = strlen(worktree_branch);
+		len += worktree_branch_len;
+	}
+	if (remote_head) {
+		remote_head_len = strlen(remote_head);
+		len += remote_head_len;
+	}
+
+	if (len >= MAX_IMSGSIZE - IMSG_HEADER_SIZE) {
+		close(fd);
+		return got_error(GOT_ERR_NO_SPACE);
+	}
+
+	wbuf = imsg_create(ibuf, GOT_IMSG_FETCH_REQUEST, 0, 0, len);
+	if (wbuf == NULL)
+		return got_error_from_errno("imsg_create FETCH_HAVE_REF");
 
 	memset(&fetchreq, 0, sizeof(fetchreq));
+	fetchreq.no_head = no_head;
 	fetchreq.fetch_all_branches = fetch_all_branches;
 	fetchreq.list_refs_only = list_refs_only;
 	fetchreq.verbosity = verbosity;
+	if (worktree_branch != NULL)
+		fetchreq.worktree_branch_len = worktree_branch_len;
+	if (remote_head != NULL)
+		fetchreq.remote_head_len = remote_head_len;
 	TAILQ_FOREACH(pe, have_refs, entry)
 		fetchreq.n_have_refs++;
 	TAILQ_FOREACH(pe, wanted_branches, entry)
 		fetchreq.n_wanted_branches++;
 	TAILQ_FOREACH(pe, wanted_refs, entry)
 		fetchreq.n_wanted_refs++;
-	len = sizeof(struct got_imsg_fetch_request);
-	if (len >= MAX_IMSGSIZE - IMSG_HEADER_SIZE) {
-		close(fd);
-		return got_error(GOT_ERR_NO_SPACE);
+	if (imsg_add(wbuf, &fetchreq, sizeof(fetchreq)) == -1)
+		return got_error_from_errno("imsg_add FETCH_REQUEST");
+	if (worktree_branch) {
+		if (imsg_add(wbuf, worktree_branch, worktree_branch_len) == -1)
+			return got_error_from_errno("imsg_add FETCH_REQUEST");
 	}
-
-	if (imsg_compose(ibuf, GOT_IMSG_FETCH_REQUEST, 0, 0, fd,
-	    &fetchreq, sizeof(fetchreq)) == -1)
-		return got_error_from_errno(
-		    "imsg_compose FETCH_SERVER_PROGRESS");
+	if (remote_head) {
+		if (imsg_add(wbuf, remote_head, remote_head_len) == -1)
+			return got_error_from_errno("imsg_add FETCH_REQUEST");
+	}
+	wbuf->fd = fd;
+	imsg_close(ibuf, wbuf);
 
 	err = flush_imsg(ibuf);
 	if (err) {
@@ -578,7 +606,7 @@ got_privsep_send_fetch_req(struct imsgbuf *ibuf, int fd,
 			return got_error_from_errno("imsg_create FETCH_HAVE_REF");
 
 		/* Keep in sync with struct got_imsg_fetch_have_ref! */
-		if (imsg_add(wbuf, id->sha1, sizeof(id->sha1)) == -1)
+		if (imsg_add(wbuf, id, sizeof(*id)) == -1)
 			return got_error_from_errno("imsg_add FETCH_HAVE_REF");
 		if (imsg_add(wbuf, &name_len, sizeof(name_len)) == -1)
 			return got_error_from_errno("imsg_add FETCH_HAVE_REF");
@@ -736,7 +764,7 @@ got_privsep_recv_fetch_progress(int *done, struct got_object_id **id,
 		}
 		break;
 	case GOT_IMSG_FETCH_REF:
-		if (datalen <= SHA1_DIGEST_LENGTH) {
+		if (datalen <= sizeof(**id)) {
 			err = got_error(GOT_ERR_PRIVSEP_MSG);
 			break;
 		}
@@ -745,9 +773,9 @@ got_privsep_recv_fetch_progress(int *done, struct got_object_id **id,
 			err = got_error_from_errno("malloc");
 			break;
 		}
-		memcpy((*id)->sha1, imsg.data, SHA1_DIGEST_LENGTH);
-		*refname = strndup(imsg.data + SHA1_DIGEST_LENGTH,
-		    datalen - SHA1_DIGEST_LENGTH);
+		memcpy(*id, imsg.data, sizeof(**id));
+		*refname = strndup(imsg.data + sizeof(**id),
+		    datalen - sizeof(**id));
 		if (*refname == NULL) {
 			err = got_error_from_errno("strndup");
 			break;
@@ -816,7 +844,7 @@ send_send_ref(const char *name, size_t name_len, struct got_object_id *id,
 		return got_error_from_errno("imsg_create SEND_REF");
 
 	/* Keep in sync with struct got_imsg_send_ref! */
-	if (imsg_add(wbuf, id->sha1, sizeof(id->sha1)) == -1)
+	if (imsg_add(wbuf, id, sizeof(*id)) == -1)
 		return got_error_from_errno("imsg_add SEND_REF");
 	if (imsg_add(wbuf, &delete, sizeof(delete)) == -1)
 		return got_error_from_errno("imsg_add SEND_REF");
@@ -918,7 +946,7 @@ got_privsep_recv_send_remote_refs(struct got_pathlist_head *remote_refs,
 				err = got_error_from_errno("malloc");
 				goto done;
 			}
-			memcpy(id->sha1, iremote_ref.id, SHA1_DIGEST_LENGTH);
+			memcpy(id, &iremote_ref.id, sizeof(*id));
 			refname = strndup(imsg.data + sizeof(iremote_ref),
 			    datalen - sizeof(iremote_ref));
 			if (refname == NULL) {
@@ -1118,7 +1146,7 @@ got_privsep_get_imsg_obj(struct got_object **obj, struct imsg *imsg,
 	if (*obj == NULL)
 		return got_error_from_errno("calloc");
 
-	memcpy((*obj)->id.sha1, iobj->id, SHA1_DIGEST_LENGTH);
+	memcpy(&(*obj)->id, &iobj->id, sizeof(iobj->id));
 	(*obj)->type = iobj->type;
 	(*obj)->flags = iobj->flags;
 	(*obj)->hdrlen = iobj->hdrlen;
@@ -1203,15 +1231,14 @@ got_privsep_send_commit(struct imsgbuf *ibuf, struct got_commit_object *commit)
 	size_t logmsg_len = strlen(commit->logmsg);
 
 	total = sizeof(*icommit) + author_len + committer_len +
-	    commit->nparents * SHA1_DIGEST_LENGTH;
+	    commit->nparents * sizeof(struct got_object_id);
 
 	buf = malloc(total);
 	if (buf == NULL)
 		return got_error_from_errno("malloc");
 
 	icommit = (struct got_imsg_commit_object *)buf;
-	memcpy(icommit->tree_id, commit->tree_id->sha1,
-	    sizeof(icommit->tree_id));
+	memcpy(&icommit->tree_id, commit->tree_id, sizeof(icommit->tree_id));
 	icommit->author_len = author_len;
 	icommit->author_time = commit->author_time;
 	icommit->author_gmtoff = commit->author_gmtoff;
@@ -1227,8 +1254,8 @@ got_privsep_send_commit(struct imsgbuf *ibuf, struct got_commit_object *commit)
 	memcpy(buf + len, commit->committer, committer_len);
 	len += committer_len;
 	STAILQ_FOREACH(qid, &commit->parent_ids, entry) {
-		memcpy(buf + len, &qid->id, SHA1_DIGEST_LENGTH);
-		len += SHA1_DIGEST_LENGTH;
+		memcpy(buf + len, &qid->id, sizeof(qid->id));
+		len += sizeof(qid->id);
 	}
 
 	if (imsg_compose(ibuf, GOT_IMSG_COMMIT, 0, 0, -1, buf, len) == -1) {
@@ -1263,7 +1290,7 @@ get_commit_from_imsg(struct got_commit_object **commit,
 	icommit = imsg->data;
 	if (datalen != sizeof(*icommit) + icommit->author_len +
 	    icommit->committer_len +
-	    icommit->nparents * SHA1_DIGEST_LENGTH)
+	    icommit->nparents * sizeof(struct got_object_id))
 		return got_error(GOT_ERR_PRIVSEP_LEN);
 
 	if (icommit->nparents < 0)
@@ -1276,8 +1303,8 @@ get_commit_from_imsg(struct got_commit_object **commit,
 		return got_error_from_errno(
 		    "got_object_commit_alloc_partial");
 
-	memcpy((*commit)->tree_id->sha1, icommit->tree_id,
-	    SHA1_DIGEST_LENGTH);
+	memcpy((*commit)->tree_id, &icommit->tree_id,
+	    sizeof(icommit->tree_id));
 	(*commit)->author_time = icommit->author_time;
 	(*commit)->author_gmtoff = icommit->author_gmtoff;
 	(*commit)->committer_time = icommit->committer_time;
@@ -1342,7 +1369,7 @@ get_commit_from_imsg(struct got_commit_object **commit,
 		if (err)
 			break;
 		memcpy(&qid->id, imsg->data + len +
-		    i * SHA1_DIGEST_LENGTH, sizeof(qid->id));
+		    i * sizeof(qid->id), sizeof(qid->id));
 		STAILQ_INSERT_TAIL(&(*commit)->parent_ids, qid, entry);
 		(*commit)->nparents++;
 	}
@@ -1407,7 +1434,7 @@ send_tree_entries_batch(struct imsgbuf *ibuf,
 	for (i = idx0; i <= idxN; i++) {
 		struct got_parsed_tree_entry *pte = &entries[i];
 
-		/* Keep in sync with struct got_imsg_tree_object definition! */
+		/* Keep in sync with struct got_imsg_tree_entry definition! */
 		if (imsg_add(wbuf, pte->id, SHA1_DIGEST_LENGTH) == -1)
 			return got_error_from_errno("imsg_add TREE_ENTRY");
 		if (imsg_add(wbuf, &pte->mode, sizeof(pte->mode)) == -1)
@@ -1436,8 +1463,7 @@ send_tree_entries(struct imsgbuf *ibuf, struct got_parsed_tree_entry *entries,
 	i = 0;
 	for (j = 0; j < nentries; j++) {
 		struct got_parsed_tree_entry *pte = &entries[j];
-		size_t len = SHA1_DIGEST_LENGTH + sizeof(pte->mode) +
-		    sizeof(pte->namelen) + pte->namelen;
+		size_t len = sizeof(struct got_imsg_tree_entry) + pte->namelen;
 
 		if (j > 0 &&
 		    entries_len + len > MAX_IMSGSIZE - IMSG_HEADER_SIZE) {
@@ -1768,7 +1794,7 @@ got_privsep_send_tag(struct imsgbuf *ibuf, struct got_tag_object *tag)
 		return got_error_from_errno("malloc");
 
 	itag = (struct got_imsg_tag_object *)buf;
-	memcpy(itag->id, tag->id.sha1, sizeof(itag->id));
+	memcpy(&itag->id, &tag->id, sizeof(itag->id));
 	itag->obj_type = tag->obj_type;
 	itag->tag_len = tag_len;
 	itag->tagger_len = tagger_len;
@@ -1839,7 +1865,7 @@ got_privsep_recv_tag(struct got_tag_object **tag, struct imsgbuf *ibuf)
 			break;
 		}
 
-		memcpy((*tag)->id.sha1, itag->id, SHA1_DIGEST_LENGTH);
+		memcpy(&(*tag)->id, &itag->id, sizeof(itag->id));
 
 		(*tag)->tag = strndup(imsg.data + len, itag->tag_len);
 		if ((*tag)->tag == NULL) {
@@ -1957,7 +1983,7 @@ got_privsep_send_packed_obj_req(struct imsgbuf *ibuf, int idx,
 
 	memset(&iobj, 0, sizeof(iobj));
 	iobj.idx = idx;
-	memcpy(iobj.id, id->sha1, sizeof(iobj.id));
+	memcpy(&iobj.id, id, sizeof(iobj.id));
 
 	if (imsg_compose(ibuf, GOT_IMSG_PACKED_OBJECT_REQUEST, 0, 0, -1,
 	    &iobj, sizeof(iobj)) == -1)
@@ -1975,7 +2001,7 @@ got_privsep_send_packed_raw_obj_req(struct imsgbuf *ibuf, int idx,
 
 	memset(&iobj, 0, sizeof(iobj));
 	iobj.idx = idx;
-	memcpy(iobj.id, id->sha1, sizeof(iobj.id));
+	memcpy(&iobj.id, id, sizeof(iobj.id));
 
 	if (imsg_compose(ibuf, GOT_IMSG_PACKED_RAW_OBJECT_REQUEST, 0, 0, -1,
 	    &iobj, sizeof(iobj)) == -1)
@@ -2103,6 +2129,60 @@ got_privsep_recv_gitconfig_str(char **str, struct imsgbuf *ibuf)
 	return err;
 }
 
+const struct got_error *
+got_privsep_recv_gitconfig_pair(char **key, char **val, struct imsgbuf *ibuf)
+{
+	const struct got_error *err = NULL;
+	struct got_imsg_gitconfig_pair p;
+	struct imsg imsg;
+	size_t datalen;
+	uint8_t *data;
+
+	*key = *val = NULL;
+
+	err = got_privsep_recv_imsg(&imsg, ibuf, 0);
+	if (err)
+		return err;
+
+	data = imsg.data;
+	datalen = imsg.hdr.len - IMSG_HEADER_SIZE;
+
+	if (imsg.hdr.type != GOT_IMSG_GITCONFIG_PAIR) {
+		err = got_error(GOT_ERR_PRIVSEP_MSG);
+		goto done;
+	}
+
+	if (datalen < sizeof(p)) {
+		err = got_error(GOT_ERR_PRIVSEP_LEN);
+		goto done;
+	}
+
+	memcpy(&p, data, sizeof(p));
+	data += sizeof(p);
+
+	if (datalen != sizeof(p) + p.klen + p.vlen) {
+		err = got_error(GOT_ERR_PRIVSEP_LEN);
+		goto done;
+	}
+
+	*key = strndup(data, p.klen);
+	if (*key == NULL) {
+		err = got_error_from_errno("strndup");
+		goto done;
+	}
+	data += p.klen;
+
+	*val = strndup(data, p.vlen);
+	if (*val == NULL) {
+		err = got_error_from_errno("strndup");
+		goto done;
+	}
+
+done:
+	imsg_free(&imsg);
+	return err;
+}
+
 const struct got_error *
 got_privsep_recv_gitconfig_int(int *val, struct imsgbuf *ibuf)
 {
@@ -2601,6 +2681,7 @@ got_privsep_recv_traversed_commits(struct got_commit_object **changed_commit,
 	const struct got_error *err = NULL;
 	struct imsg imsg;
 	struct got_imsg_traversed_commits *icommits;
+	struct got_object_id *ids;
 	size_t datalen;
 	int i, done = 0;
 
@@ -2617,18 +2698,18 @@ got_privsep_recv_traversed_commits(struct got_commit_object **changed_commit,
 		case GOT_IMSG_TRAVERSED_COMMITS:
 			icommits = imsg.data;
 			if (datalen != sizeof(*icommits) +
-			    icommits->ncommits * SHA1_DIGEST_LENGTH) {
+			    icommits->ncommits * sizeof(*ids)) {
 				err = got_error(GOT_ERR_PRIVSEP_LEN);
 				break;
 			}
+			ids = imsg.data + sizeof(*icommits);
 			for (i = 0; i < icommits->ncommits; i++) {
 				struct got_object_qid *qid;
-				uint8_t *sha1 = (uint8_t *)imsg.data +
-				    sizeof(*icommits) + i * SHA1_DIGEST_LENGTH;
+
 				err = got_object_qid_alloc_partial(&qid);
 				if (err)
 					break;
-				memcpy(qid->id.sha1, sha1, SHA1_DIGEST_LENGTH);
+				memcpy(&qid->id, &ids[i], sizeof(ids[i]));
 				STAILQ_INSERT_TAIL(commit_ids, qid, entry);
 
 				/* The last commit may contain a change. */
@@ -2954,7 +3035,7 @@ got_privsep_send_raw_delta_req(struct imsgbuf *ibuf, int idx,
 
 	memset(&dreq, 0, sizeof(dreq));
 	dreq.idx = idx;
-	memcpy(dreq.id, id->sha1, SHA1_DIGEST_LENGTH);
+	memcpy(&dreq.id, id, sizeof(dreq.id));
 
 	if (imsg_compose(ibuf, GOT_IMSG_RAW_DELTA_REQUEST, 0, 0, -1,
 	    &dreq, sizeof(dreq)) == -1)
@@ -2984,7 +3065,7 @@ got_privsep_send_raw_delta(struct imsgbuf *ibuf, uint64_t base_size,
 	idelta.delta_compressed_size = delta_compressed_size;
 	idelta.delta_offset = delta_offset;
 	idelta.delta_out_offset = delta_out_offset;
-	memcpy(idelta.base_id, base_id->sha1, SHA1_DIGEST_LENGTH);
+	memcpy(&idelta.base_id, &base_id, sizeof(idelta.base_id));
 
 	ret = imsg_compose(ibuf, GOT_IMSG_RAW_DELTA, 0, 0, -1,
 	    &idelta, sizeof(idelta));
@@ -2997,7 +3078,8 @@ got_privsep_send_raw_delta(struct imsgbuf *ibuf, uint64_t base_size,
 const struct got_error *
 got_privsep_recv_raw_delta(uint64_t *base_size, uint64_t *result_size,
     off_t *delta_size, off_t *delta_compressed_size, off_t *delta_offset,
-    off_t *delta_out_offset, struct got_object_id **base_id, struct imsgbuf *ibuf)
+    off_t *delta_out_offset, struct got_object_id **base_id,
+    struct imsgbuf *ibuf)
 {
 	const struct got_error *err = NULL;
 	struct imsg imsg;
@@ -3036,7 +3118,7 @@ got_privsep_recv_raw_delta(uint64_t *base_size, uint64_t *result_size,
 			err = got_error_from_errno("malloc");
 			break;
 		}
-		memcpy((*base_id)->sha1, delta->base_id, SHA1_DIGEST_LENGTH);
+		memcpy(*base_id, &delta->base_id, sizeof(**base_id));
 		break;
 	default:
 		err = got_error(GOT_ERR_PRIVSEP_MSG);
@@ -3301,7 +3383,7 @@ got_privsep_send_painting_request(struct imsgbuf *ibuf, int idx,
 	struct got_imsg_commit_painting_request ireq;
 
 	memset(&ireq, 0, sizeof(ireq));
-	memcpy(ireq.id, id->sha1, sizeof(ireq.id));
+	memcpy(&ireq.id, id, sizeof(ireq.id));
 	ireq.idx = idx;
 	ireq.color = color;
 
