@@ -49,7 +49,7 @@
 #include "got_lib_repository.h"
 #include "got_lib_ratelimit.h"
 #include "got_lib_pack_create.h"
-#include "got_lib_sha1.h"
+#include "got_lib_hash.h"
 #include "got_lib_lockfile.h"
 
 #ifndef nitems
@@ -141,7 +141,7 @@ const struct got_error *
 got_repo_pack_objects(FILE **packfile, struct got_object_id **pack_hash,
     struct got_reflist_head *include_refs,
     struct got_reflist_head *exclude_refs, struct got_repository *repo,
-    int loose_obj_only,
+    int loose_obj_only, int force_refdelta,
     got_pack_progress_cb progress_cb, void *progress_arg,
     got_cancel_cb cancel_cb, void *cancel_arg)
 {
@@ -167,7 +167,7 @@ got_repo_pack_objects(FILE **packfile, struct got_object_id **pack_hash,
 	if (err)
 		goto done;
 
-	if (fchmod(packfd, GOT_DEFAULT_FILE_MODE) != 0) {
+	if (fchmod(packfd, GOT_DEFAULT_PACK_MODE) == -1) {
 		err = got_error_from_errno2("fchmod", tmpfile_path);
 		goto done;
 	}
@@ -205,8 +205,9 @@ got_repo_pack_objects(FILE **packfile, struct got_object_id **pack_hash,
 	}
 
 	err = got_pack_create((*pack_hash)->sha1, packfd, delta_cache,
-	    theirs, ntheirs, ours, nours, repo, loose_obj_only, 0,
-	    progress_cb, progress_arg, &rl, cancel_cb, cancel_arg);
+	    theirs, ntheirs, ours, nours, repo, loose_obj_only,
+	    0, force_refdelta, progress_cb, progress_arg, &rl,
+	    cancel_cb, cancel_arg);
 	if (err)
 		goto done;
 
@@ -297,7 +298,7 @@ got_repo_index_pack(FILE *packfile, struct got_object_id *pack_hash,
 	free(path);
 	if (err)
 		goto done;
-	if (fchmod(idxfd, GOT_DEFAULT_FILE_MODE) != 0) {
+	if (fchmod(idxfd, GOT_DEFAULT_PACK_MODE) == -1) {
 		err = got_error_from_errno2("fchmod", tmpidxpath);
 		goto done;
 	}
@@ -480,7 +481,7 @@ got_repo_find_pack(FILE **packfile, struct got_object_id **pack_hash,
 		goto done;
 	}
 	*dot = '\0';
-	if (!got_parse_sha1_digest(id.sha1, p)) {
+	if (!got_parse_object_id(&id, p, GOT_HASH_SHA1)) {
 		err = got_error_fmt(GOT_ERR_BAD_PATH,
 		    "'%s' is not a valid pack file name",
 		    packfile_name);
@@ -679,8 +680,8 @@ get_loose_object_ids(struct got_object_idset **loose_ids, off_t *ondisk_size,
 				goto done;
 			}
 
-			memset(&id, 0, sizeof(id));
-			if (!got_parse_sha1_digest(id.sha1, id_str)) {
+			if (!got_parse_object_id(&id, id_str,
+			    GOT_HASH_SHA1)) {
 				free(id_str);
 				continue;
 			}
