@@ -42,7 +42,7 @@
 #include "got_fetch.h"
 #include "got_reference.h"
 
-#include "got_lib_sha1.h"
+#include "got_lib_hash.h"
 #include "got_lib_delta.h"
 #include "got_lib_object.h"
 #include "got_lib_object_parse.h"
@@ -71,7 +71,7 @@ static const struct got_capability got_capabilities[] = {
 
 static void
 match_remote_ref(struct got_pathlist_head *have_refs,
-    struct got_object_id *my_id, char *refname)
+    struct got_object_id *my_id, const char *refname)
 {
 	struct got_pathlist_entry *pe;
 
@@ -281,7 +281,7 @@ send_fetch_ref(struct imsgbuf *ibuf, struct got_object_id *refid,
 		return got_error_from_errno("imsg_create FETCH_REF");
 
 	/* Keep in sync with struct got_imsg_fetch_ref definition! */
-	if (imsg_add(wbuf, refid->sha1, SHA1_DIGEST_LENGTH) == -1)
+	if (imsg_add(wbuf, refid, sizeof(*refid)) == -1)
 		return got_error_from_errno("imsg_add FETCH_REF");
 	if (imsg_add(wbuf, refname, reflen) == -1)
 		return got_error_from_errno("imsg_add FETCH_REF");
@@ -291,12 +291,50 @@ send_fetch_ref(struct imsgbuf *ibuf, struct got_object_id *refid,
 	return got_privsep_flush_imsg(ibuf);
 }
 
+static const struct got_error *
+fetch_ref(struct imsgbuf *ibuf, struct got_pathlist_head *have_refs,
+    struct got_object_id *have, struct got_object_id *want,
+    const char *refname, const char *id_str)
+{
+	const struct got_error *err;
+	char *theirs = NULL, *mine = NULL;
+
+	if (!got_parse_object_id(want, id_str, GOT_HASH_SHA1)) {
+		err = got_error(GOT_ERR_BAD_OBJ_ID_STR);
+		goto done;
+	}
+
+	match_remote_ref(have_refs, have, refname);
+	err = send_fetch_ref(ibuf, want, refname);
+	if (err)
+		goto done;
+
+	if (chattygot)
+		fprintf(stderr, "%s: %s will be fetched\n",
+		    getprogname(), refname);
+	if (chattygot > 1) {
+		err = got_object_id_str(&theirs, want);
+		if (err)
+			goto done;
+		err = got_object_id_str(&mine, have);
+		if (err)
+			goto done;
+		fprintf(stderr, "%s: remote: %s\n%s: local:  %s\n",
+		    getprogname(), theirs, getprogname(), mine);
+	}
+done:
+	free(theirs);
+	free(mine);
+	return err;
+}
+
 static const struct got_error *
 fetch_pack(int fd, int packfd, uint8_t *pack_sha1,
     struct got_pathlist_head *have_refs, int fetch_all_branches,
     struct got_pathlist_head *wanted_branches,
     struct got_pathlist_head *wanted_refs, int list_refs_only,
-    struct imsgbuf *ibuf)
+    const char *worktree_branch, const char *remote_head,
+    int no_head, struct imsgbuf *ibuf)
 {
 	const struct got_error *err = NULL;
 	char buf[GOT_PKT_MAX];
@@ -305,7 +343,7 @@ fetch_pack(int fd, int packfd, uint8_t *pack_sha1,
 	int is_firstpkt = 1, nref = 0, refsz = 16;
 	int i, n, nwant = 0, nhave = 0, acked = 0;
 	off_t packsz = 0, last_reported_packsz = 0;
-	char *id_str = NULL, *refname = NULL;
+	char *id_str = NULL, *default_id_str = NULL, *refname = NULL;
 	char *server_capabilities = NULL, *my_capabilities = NULL;
 	const char *default_branch = NULL;
 	struct got_pathlist_head symrefs;
@@ -346,6 +384,25 @@ fetch_pack(int fd, int packfd, uint8_t *pack_sha1,
 		    &server_capabilities, buf, n);
 		if (err)
 			goto done;
+
+		if (refsz == nref + 1) {
+			struct got_object_id *h, *w;
+
+			refsz *= 2;
+			h = reallocarray(have, refsz, sizeof(have[0]));
+			if (h == NULL) {
+				err = got_error_from_errno("reallocarray");
+				goto done;
+			}
+			have = h;
+			w = reallocarray(want, refsz, sizeof(want[0]));
+			if (w == NULL) {
+				err = got_error_from_errno("reallocarray");
+				goto done;
+			}
+			want = w;
+		}
+
 		if (is_firstpkt) {
 			if (chattygot && server_capabilities[0] != '\0')
 				fprintf(stderr, "%s: server capabilities: %s\n",
@@ -383,107 +440,112 @@ fetch_pack(int fd, int packfd, uint8_t *pack_sha1,
 			}
 			continue;
 		}
-
-		if (strncmp(refname, "refs/heads/", 11) == 0) {
-			if (fetch_all_branches || list_refs_only) {
-				found_branch = 1;
-			} else if (!TAILQ_EMPTY(wanted_branches)) {
-				TAILQ_FOREACH(pe, wanted_branches, entry) {
-					if (match_branch(refname, pe->path))
-						break;
-				}
-				if (pe == NULL) {
-					if (chattygot) {
-						fprintf(stderr,
-						    "%s: ignoring %s\n",
-						    getprogname(), refname);
-					}
-					continue;
-				}
-				found_branch = 1;
-			} else if (default_branch != NULL) {
-				if (!match_branch(refname, default_branch)) {
-					if (chattygot) {
-						fprintf(stderr,
-						    "%s: ignoring %s\n",
-						    getprogname(), refname);
-					}
-					continue;
-				}
-				found_branch = 1;
-			}
-		} else if (strncmp(refname, "refs/tags/", 10) != 0) {
-			if (!TAILQ_EMPTY(wanted_refs)) {
-				TAILQ_FOREACH(pe, wanted_refs, entry) {
-					if (match_wanted_ref(refname, pe->path))
-						break;
-				}
-				if (pe == NULL) {
-					if (chattygot) {
-						fprintf(stderr,
-						    "%s: ignoring %s\n",
-						    getprogname(), refname);
-					}
-					continue;
-				}
-				found_branch = 1;
-			} else if (!list_refs_only) {
-				if (chattygot) {
-					fprintf(stderr, "%s: ignoring %s\n",
-					    getprogname(), refname);
-				}
-				continue;
-			}
-		}
-
-		if (refsz == nref + 1) {
-			refsz *= 2;
-			have = reallocarray(have, refsz, sizeof(have[0]));
-			if (have == NULL) {
-				err = got_error_from_errno("reallocarray");
-				goto done;
-			}
-			want = reallocarray(want, refsz, sizeof(want[0]));
-			if (want == NULL) {
-				err = got_error_from_errno("reallocarray");
+		if (default_branch && default_id_str == NULL &&
+		    strcmp(refname, default_branch) == 0) {
+			default_id_str = strdup(id_str);
+			if (default_id_str == NULL) {
+				err = got_error_from_errno("strdup");
 				goto done;
 			}
 		}
-		if (!got_parse_sha1_digest(want[nref].sha1, id_str)) {
-			err = got_error(GOT_ERR_BAD_OBJ_ID_STR);
-			goto done;
-		}
-		match_remote_ref(have_refs, &have[nref], refname);
-		err = send_fetch_ref(ibuf, &want[nref], refname);
-		if (err)
-			goto done;
 
-		if (chattygot)
-			fprintf(stderr, "%s: %s will be fetched\n",
-			    getprogname(), refname);
-		if (chattygot > 1) {
-			char *theirs, *mine;
-			err = got_object_id_str(&theirs, &want[nref]);
+		if (list_refs_only || strncmp(refname, "refs/tags/", 10) == 0) {
+			err = fetch_ref(ibuf, have_refs, &have[nref],
+			    &want[nref], refname, id_str);
 			if (err)
 				goto done;
-			err = got_object_id_str(&mine, &have[nref]);
-			if (err) {
-				free(theirs);
-				goto done;
+			nref++;
+		} else if (strncmp(refname, "refs/heads/", 11) == 0) {
+			if (fetch_all_branches) {
+				err = fetch_ref(ibuf, have_refs, &have[nref],
+				    &want[nref], refname, id_str);
+				if (err)
+					goto done;
+				nref++;
+				found_branch = 1;
+				continue;
+			}
+			TAILQ_FOREACH(pe, wanted_branches, entry) {
+				if (match_branch(refname, pe->path))
+					break;
+			}
+			if (pe != NULL || (worktree_branch != NULL &&
+			    match_branch(refname, worktree_branch))) {
+				err = fetch_ref(ibuf, have_refs, &have[nref],
+				    &want[nref], refname, id_str);
+				if (err)
+					goto done;
+				nref++;
+				found_branch = 1;
+			} else if (chattygot) {
+				fprintf(stderr, "%s: ignoring %s\n",
+				    getprogname(), refname);
+			}
+		} else {
+			TAILQ_FOREACH(pe, wanted_refs, entry) {
+				if (match_wanted_ref(refname, pe->path))
+					break;
+			}
+			if (pe != NULL) {
+				err = fetch_ref(ibuf, have_refs, &have[nref],
+				    &want[nref], refname, id_str);
+				if (err)
+					goto done;
+				nref++;
+			} else if (chattygot) {
+				fprintf(stderr, "%s: ignoring %s\n",
+				    getprogname(), refname);
 			}
-			fprintf(stderr, "%s: remote: %s\n%s: local:  %s\n",
-			    getprogname(), theirs, getprogname(), mine);
-			free(theirs);
-			free(mine);
 		}
-		nref++;
 	}
 
 	if (list_refs_only)
 		goto done;
 
-	/* Abort if we haven't found any branch to fetch. */
-	if (!found_branch) {
+	/*
+	 * If -b was not used and either none of the requested branches
+	 * (got.conf, worktree) were found or the client already has the
+	 * remote HEAD symref but its target changed, fetch remote's HEAD.
+	 */
+	if (!no_head && default_branch && default_id_str &&
+	    strncmp(default_branch, "refs/heads/", 11) == 0) {
+		int remote_head_changed = 0;
+
+		if (remote_head) {
+			if (strcmp(remote_head, default_branch + 11) != 0)
+				remote_head_changed = 1;
+		}
+
+		if (!found_branch || remote_head_changed) {
+			err = fetch_ref(ibuf, have_refs, &have[nref],
+			    &want[nref], default_branch, default_id_str);
+			if (err)
+				goto done;
+			nref++;
+		}
+	}
+
+	/* Abort if we haven't found anything to fetch. */
+	if (nref == 0) {
+		struct got_pathlist_entry *pe;
+		static char msg[PATH_MAX + 33];
+
+		pe = TAILQ_FIRST(wanted_branches);
+		if (pe) {
+			snprintf(msg, sizeof(msg),
+			    "branch \"%s\" not found on server", pe->path);
+			err = got_error_msg(GOT_ERR_FETCH_NO_BRANCH, msg);
+			goto done;
+		}
+
+		pe = TAILQ_FIRST(wanted_refs);
+		if (pe) {
+			snprintf(msg, sizeof(msg),
+			    "reference \"%s\" not found on server", pe->path);
+			err = got_error_msg(GOT_ERR_FETCH_NO_BRANCH, msg);
+			goto done;
+		}
+
 		err = got_error(GOT_ERR_FETCH_NO_BRANCH);
 		goto done;
 	}
@@ -526,6 +588,11 @@ fetch_pack(int fd, int packfd, uint8_t *pack_sha1,
 		nhave++;
 	}
 
+	n = strlcpy(buf, "done\n", sizeof(buf));
+	err = got_pkt_writepkt(fd, buf, n, chattygot);
+	if (err)
+		goto done;
+
 	while (nhave > 0 && !acked) {
 		struct got_object_id common_id;
 
@@ -538,8 +605,12 @@ fetch_pack(int fd, int packfd, uint8_t *pack_sha1,
 			goto done;
 		}
 		if (n >= 4 && strncmp(buf, "NAK\n", 4) == 0) {
-			/* Server has not located our objects yet. */
-			continue;
+			/*
+			 * Server could not find a common ancestor.
+			 * Perhaps it is an out-of-date mirror, or there
+			 * is a repository with unrelated history.
+			 */
+			break;
 		}
 		if (n < 4 + SHA1_DIGEST_STRING_LENGTH ||
 		    strncmp(buf, "ACK ", 4) != 0) {
@@ -547,7 +618,8 @@ fetch_pack(int fd, int packfd, uint8_t *pack_sha1,
 			    "unexpected message from server");
 			goto done;
 		}
-		if (!got_parse_sha1_digest(common_id.sha1, buf + 4)) {
+		if (!got_parse_object_id(&common_id, buf + 4,
+		    GOT_HASH_SHA1)) {
 			err = got_error_msg(GOT_ERR_BAD_PACKET,
 			    "bad object ID in ACK packet from server");
 			goto done;
@@ -555,11 +627,6 @@ fetch_pack(int fd, int packfd, uint8_t *pack_sha1,
 		acked++;
 	}
 
-	n = strlcpy(buf, "done\n", sizeof(buf));
-	err = got_pkt_writepkt(fd, buf, n, chattygot);
-	if (err)
-		goto done;
-
 	if (nhave == 0) {
 		err = got_pkt_readpkt(&n, fd, buf, sizeof(buf), chattygot);
 		if (err)
@@ -763,6 +830,7 @@ done:
 	free(have);
 	free(want);
 	free(id_str);
+	free(default_id_str);
 	free(refname);
 	free(server_capabilities);
 	return err;
@@ -785,6 +853,7 @@ main(int argc, char **argv)
 	struct got_imsg_fetch_wanted_branch wbranch;
 	struct got_imsg_fetch_wanted_ref wref;
 	size_t datalen, i;
+	char *remote_head = NULL, *worktree_branch = NULL;
 #if 0
 	static int attached;
 	while (!attached)
@@ -835,6 +904,31 @@ main(int argc, char **argv)
 	}
 	memcpy(&fetch_req, imsg.data, sizeof(fetch_req));
 	fetchfd = imsg.fd;
+
+	if (datalen != sizeof(fetch_req) +
+	    fetch_req.worktree_branch_len + fetch_req.remote_head_len) {
+		err = got_error(GOT_ERR_PRIVSEP_LEN);
+		goto done;
+	}
+
+	if (fetch_req.worktree_branch_len != 0) {
+		worktree_branch = strndup(imsg.data +
+		    sizeof(fetch_req), fetch_req.worktree_branch_len);
+		if (worktree_branch == NULL) {
+			err = got_error_from_errno("strndup");
+			goto done;
+		}
+	}
+
+	if (fetch_req.remote_head_len != 0) {
+		remote_head = strndup(imsg.data + sizeof(fetch_req) +
+		    fetch_req.worktree_branch_len, fetch_req.remote_head_len);
+		if (remote_head == NULL) {
+			err = got_error_from_errno("strndup");
+			goto done;
+		}
+	}
+
 	imsg_free(&imsg);
 
 	if (fetch_req.verbosity > 0)
@@ -878,7 +972,7 @@ main(int argc, char **argv)
 			err = got_error_from_errno("malloc");
 			goto done;
 		}
-		memcpy(id->sha1, href.id, SHA1_DIGEST_LENGTH);
+		memcpy(id, &href.id, sizeof(*id));
 		err = got_pathlist_append(&have_refs, refname, id);
 		if (err) {
 			free(refname);
@@ -990,8 +1084,11 @@ main(int argc, char **argv)
 
 	err = fetch_pack(fetchfd, packfd, pack_sha1, &have_refs,
 	    fetch_req.fetch_all_branches, &wanted_branches,
-	    &wanted_refs, fetch_req.list_refs_only, &ibuf);
+	    &wanted_refs, fetch_req.list_refs_only,
+	    worktree_branch, remote_head, fetch_req.no_head, &ibuf);
 done:
+	free(worktree_branch);
+	free(remote_head);
 	got_pathlist_free(&have_refs, GOT_PATHLIST_FREE_ALL);
 	got_pathlist_free(&wanted_branches, GOT_PATHLIST_FREE_PATH);
 	if (fetchfd != -1 && close(fetchfd) == -1 && err == NULL)
