@@ -95,7 +95,7 @@ object_request(struct imsg *imsg, struct imsgbuf *ibuf, struct got_pack *pack,
 	if (datalen != sizeof(iobj))
 		return got_error(GOT_ERR_PRIVSEP_LEN);
 	memcpy(&iobj, imsg->data, sizeof(iobj));
-	memcpy(id.sha1, iobj.id, SHA1_DIGEST_LENGTH);
+	memcpy(&id, &iobj.id, sizeof(id));
 
 	obj = got_object_cache_get(objcache, &id);
 	if (obj) {
@@ -162,7 +162,7 @@ commit_request(struct imsg *imsg, struct imsgbuf *ibuf, struct got_pack *pack,
 	if (datalen != sizeof(iobj))
 		return got_error(GOT_ERR_PRIVSEP_LEN);
 	memcpy(&iobj, imsg->data, sizeof(iobj));
-	memcpy(id.sha1, iobj.id, SHA1_DIGEST_LENGTH);
+	memcpy(&id, &iobj.id, sizeof(id));
 
 	err = open_commit(&commit, pack, packidx, iobj.idx, &id, objcache);
 	if (err)
@@ -237,7 +237,7 @@ tree_request(struct imsg *imsg, struct imsgbuf *ibuf, struct got_pack *pack,
 	if (datalen != sizeof(iobj))
 		return got_error(GOT_ERR_PRIVSEP_LEN);
 	memcpy(&iobj, imsg->data, sizeof(iobj));
-	memcpy(id.sha1, iobj.id, SHA1_DIGEST_LENGTH);
+	memcpy(&id, &iobj.id, sizeof(id));
 
 	err = open_tree(&buf, entries, nentries, nentries_alloc,
 	    pack, packidx, iobj.idx, &id, objcache);
@@ -332,7 +332,7 @@ blob_request(struct imsg *imsg, struct imsgbuf *ibuf, struct got_pack *pack,
 	if (datalen != sizeof(iobj))
 		return got_error(GOT_ERR_PRIVSEP_LEN);
 	memcpy(&iobj, imsg->data, sizeof(iobj));
-	memcpy(id.sha1, iobj.id, SHA1_DIGEST_LENGTH);
+	memcpy(&id, &iobj.id, sizeof(id));
 
 	obj = got_object_cache_get(objcache, &id);
 	if (obj) {
@@ -393,7 +393,7 @@ tag_request(struct imsg *imsg, struct imsgbuf *ibuf, struct got_pack *pack,
 	if (datalen != sizeof(iobj))
 		return got_error(GOT_ERR_PRIVSEP_LEN);
 	memcpy(&iobj, imsg->data, sizeof(iobj));
-	memcpy(id.sha1, iobj.id, SHA1_DIGEST_LENGTH);
+	memcpy(&id, &iobj.id, sizeof(id));
 
 	obj = got_object_cache_get(objcache, &id);
 	if (obj) {
@@ -544,7 +544,8 @@ tree_path_changed(int *changed, uint8_t **buf1, uint8_t **buf2,
 			free(*buf2);
 			*buf2 = NULL;
 			err = open_tree(buf2, entries2, nentries2,
-			nentries_alloc2, pack, packidx, idx, &id2, objcache);
+			    nentries_alloc2, pack, packidx, idx, &id2,
+			    objcache);
 			pte2 = NULL;
 			if (err)
 				break;
@@ -563,7 +564,7 @@ send_traversed_commits(struct got_object_id *commit_ids, size_t ncommits,
 
 	wbuf = imsg_create(ibuf, GOT_IMSG_TRAVERSED_COMMITS, 0, 0,
 	    sizeof(struct got_imsg_traversed_commits) +
-	    ncommits * SHA1_DIGEST_LENGTH);
+	    ncommits * sizeof(commit_ids[0]));
 	if (wbuf == NULL)
 		return got_error_from_errno("imsg_create TRAVERSED_COMMITS");
 
@@ -572,7 +573,7 @@ send_traversed_commits(struct got_object_id *commit_ids, size_t ncommits,
 
 	for (i = 0; i < ncommits; i++) {
 		struct got_object_id *id = &commit_ids[i];
-		if (imsg_add(wbuf, id->sha1, SHA1_DIGEST_LENGTH) == -1) {
+		if (imsg_add(wbuf, id, sizeof(*id)) == -1) {
 			return got_error_from_errno(
 			    "imsg_add TRAVERSED_COMMITS");
 		}
@@ -617,7 +618,7 @@ commit_traversal_request(struct imsg *imsg, struct imsgbuf *ibuf,
 	if (datalen < sizeof(iobj))
 		return got_error(GOT_ERR_PRIVSEP_LEN);
 	memcpy(&iobj, imsg->data, sizeof(iobj));
-	memcpy(id.sha1, iobj.id, SHA1_DIGEST_LENGTH);
+	memcpy(&id, &iobj.id, sizeof(id));
 
 	path_len = datalen - sizeof(iobj) - 1;
 	if (path_len < 0)
@@ -657,7 +658,7 @@ commit_traversal_request(struct imsg *imsg, struct imsgbuf *ibuf,
 		}
 
 		if (sizeof(struct got_imsg_traversed_commits) +
-		    ncommits * SHA1_DIGEST_LENGTH >= max_datalen) {
+		    ncommits * sizeof(commit_ids[0]) >= max_datalen) {
 			err = send_traversed_commits(commit_ids, ncommits,
 			    ibuf);
 			if (err)
@@ -676,8 +677,7 @@ commit_traversal_request(struct imsg *imsg, struct imsgbuf *ibuf,
 			}
 			commit_ids = new;
 		}
-		memcpy(commit_ids[ncommits - 1].sha1, id.sha1,
-		    SHA1_DIGEST_LENGTH);
+		memcpy(&commit_ids[ncommits - 1], &id, sizeof(id));
 
 		pid = STAILQ_FIRST(&commit->parent_ids);
 		if (pid == NULL)
@@ -746,7 +746,7 @@ commit_traversal_request(struct imsg *imsg, struct imsgbuf *ibuf,
 		}
 
 		if (!changed) {
-			memcpy(id.sha1, pid->id.sha1, SHA1_DIGEST_LENGTH);
+			memcpy(&id, &pid->id, sizeof(id));
 			got_object_commit_close(commit);
 			commit = pcommit;
 			pcommit = NULL;
@@ -801,7 +801,7 @@ raw_object_request(struct imsg *imsg, struct imsgbuf *ibuf,
 	if (datalen != sizeof(iobj))
 		return got_error(GOT_ERR_PRIVSEP_LEN);
 	memcpy(&iobj, imsg->data, sizeof(iobj));
-	memcpy(id.sha1, iobj.id, SHA1_DIGEST_LENGTH);
+	memcpy(&id, &iobj.id, sizeof(id));
 
 	obj = got_object_cache_get(objcache, &id);
 	if (obj) {
@@ -880,7 +880,7 @@ raw_delta_request(struct imsg *imsg, struct imsgbuf *ibuf,
 	if (datalen != sizeof(req))
 		return got_error(GOT_ERR_PRIVSEP_LEN);
 	memcpy(&req, imsg->data, sizeof(req));
-	memcpy(id.sha1, req.id, SHA1_DIGEST_LENGTH);
+	memcpy(&id, &req.id, sizeof(id));
 
 	imsg->fd = -1;
 
@@ -1193,7 +1193,7 @@ enumerate_tree(int *have_all_entries, struct imsgbuf *ibuf, size_t *totlen,
 	err = got_object_qid_alloc_partial(&qid);
 	if (err)
 		return err;
-	memcpy(&qid->id.sha1, tree_id, SHA1_DIGEST_LENGTH);
+	memcpy(&qid->id, tree_id, sizeof(*tree_id));
 	qid->data = strdup(path);
 	if (qid->data == NULL) {
 		err = got_error_from_errno("strdup");
@@ -1797,7 +1797,7 @@ commit_painting_request(struct imsg *imsg, struct imsgbuf *ibuf,
 	if (datalen != sizeof(ireq))
 		return got_error(GOT_ERR_PRIVSEP_LEN);
 	memcpy(&ireq, imsg->data, sizeof(ireq));
-	memcpy(id.sha1, ireq.id, SHA1_DIGEST_LENGTH);
+	memcpy(&id, &ireq.id, sizeof(id));
 
 	err = queue_commit_id(&ids, &id, ireq.color);
 	if (err)
